@@ -1,22 +1,18 @@
 # -*- coding: utf-8 -*-
 from __future__ import with_statement
 import os
+import pep8
 import sys
 import stat
 from subprocess import Popen, PIPE
 import shutil
-from tempfile import mkdtemp
-try:
-    # The 'demandimport' breaks pyflakes and flake8._pyflakes
-    from mercurial import demandimport
-    demandimport.disable()
-except ImportError:
-    pass
+import tempfile
 try:
     from configparser import ConfigParser
 except ImportError:   # Python 2
     from ConfigParser import ConfigParser
 
+from flake8 import compat
 from flake8.engine import get_parser, get_style_guide
 from flake8.main import DEFAULT_CONFIG
 
@@ -53,16 +49,13 @@ def git_hook(complexity=-1, strict=False, ignore=None, lazy=False):
     if complexity > -1:
         options['max_complexity'] = complexity
 
-    files_modified = [f for f in files_modified if f.endswith('.py')]
+    tmpdir = tempfile.mkdtemp()
 
-    flake8_style = get_style_guide(
-        parse_argv=True,
-        config_file=DEFAULT_CONFIG,
-        **options
-        )
+    flake8_style = get_style_guide(config_file=DEFAULT_CONFIG, paths=['.'],
+                                   **options)
+    filepatterns = flake8_style.options.filename
 
     # Copy staged versions to temporary directory
-    tmpdir = mkdtemp()
     files_to_check = []
     try:
         for file_ in files_modified:
@@ -73,15 +66,21 @@ def git_hook(complexity=-1, strict=False, ignore=None, lazy=False):
             # avoid overwriting files with the same name
             dirname, filename = os.path.split(os.path.abspath(file_))
             prefix = os.path.commonprefix([dirname, tmpdir])
-            dirname = os.path.relpath(dirname, start=prefix)
+            dirname = compat.relpath(dirname, start=prefix)
             dirname = os.path.join(tmpdir, dirname)
             if not os.path.isdir(dirname):
                 os.makedirs(dirname)
-            filename = os.path.join(dirname, filename)
-            # write staged version of file to temporary directory
-            with open(filename, "wb") as fh:
-                fh.write(out)
-            files_to_check.append(filename)
+
+            # check_files() only does this check if passed a dir; so we do it
+            if ((pep8.filename_match(file_, filepatterns) and
+                 not flake8_style.excluded(file_))):
+
+                filename = os.path.join(dirname, filename)
+                files_to_check.append(filename)
+                # write staged version of file to temporary directory
+                with open(filename, "wb") as fh:
+                    fh.write(out)
+
         # Run the checks
         report = flake8_style.check_files(files_to_check)
     # remove temporary directory
@@ -103,9 +102,7 @@ def hg_hook(ui, repo, **kwargs):
     complexity = ui.config('flake8', 'complexity', default=-1)
     strict = ui.configbool('flake8', 'strict', default=True)
     ignore = ui.config('flake8', 'ignore', default=None)
-    config = ui.config('flake8', 'config', default=True)
-    if config is True:
-        config = DEFAULT_CONFIG
+    config = ui.config('flake8', 'config', default=DEFAULT_CONFIG)
 
     paths = _get_files(repo, **kwargs)
 
@@ -117,7 +114,7 @@ def hg_hook(ui, repo, **kwargs):
     if complexity > -1:
         options['max_complexity'] = complexity
 
-    flake8_style = get_style_guide(parse_argv=True, config_file=config,
+    flake8_style = get_style_guide(config_file=config, paths=['.'],
                                    **options)
     report = flake8_style.check_files(paths)
 
@@ -134,12 +131,11 @@ def run(command, raw_output=False, decode=True):
     # endswith to be given a bytes object or a tuple of bytes but not native
     # string objects. This is simply less mysterious than using b'.py' in the
     # endswith method. That should work but might still fail horribly.
-    if hasattr(stdout, 'decode'):
-        if decode:
-            stdout = stdout.decode()
-    if hasattr(stderr, 'decode'):
-        if decode:
-            stderr = stderr.decode()
+    if decode:
+        if hasattr(stdout, 'decode'):
+            stdout = stdout.decode('utf-8')
+        if hasattr(stderr, 'decode'):
+            stderr = stderr.decode('utf-8')
     if not raw_output:
         stdout = [line.strip() for line in stdout.splitlines()]
         stderr = [line.strip() for line in stderr.splitlines()]
@@ -159,27 +155,74 @@ def _get_files(repo, **kwargs):
 
 
 def find_vcs():
-    _, git_dir, _ = run('git rev-parse --git-dir')
-    if git_dir and os.path.isdir(git_dir[0]):
-        if not os.path.isdir(os.path.join(git_dir[0], 'hooks')):
-            os.mkdir(os.path.join(git_dir[0], 'hooks'))
-        return os.path.join(git_dir[0], 'hooks', 'pre-commit')
-    _, hg_dir, _ = run('hg root')
-    if hg_dir and os.path.isdir(hg_dir[0]):
-        return os.path.join(hg_dir[0], '.hg', 'hgrc')
+    try:
+        _, git_dir, _ = run('git rev-parse --git-dir')
+    except OSError:
+        pass
+    else:
+        if git_dir and os.path.isdir(git_dir[0]):
+            if not os.path.isdir(os.path.join(git_dir[0], 'hooks')):
+                os.mkdir(os.path.join(git_dir[0], 'hooks'))
+            return os.path.join(git_dir[0], 'hooks', 'pre-commit')
+    try:
+        _, hg_dir, _ = run('hg root')
+    except OSError:
+        pass
+    else:
+        if hg_dir and os.path.isdir(hg_dir[0]):
+            return os.path.join(hg_dir[0], '.hg', 'hgrc')
     return ''
 
 
+def get_git_config(option, opt_type='', convert_type=True):
+    # type can be --bool, --int or an empty string
+    _, git_cfg_value, _ = run('git config --get %s %s' % (opt_type, option),
+                              raw_output=True)
+    git_cfg_value = git_cfg_value.strip()
+    if not convert_type:
+        return git_cfg_value
+    if opt_type == '--bool':
+        git_cfg_value = git_cfg_value.lower() == 'true'
+    elif git_cfg_value and opt_type == '--int':
+        git_cfg_value = int(git_cfg_value)
+    return git_cfg_value
+
+
+_params = {
+    'FLAKE8_COMPLEXITY': '--int',
+    'FLAKE8_STRICT': '--bool',
+    'FLAKE8_IGNORE': '',
+    'FLAKE8_LAZY': '--bool',
+}
+
+
+def get_git_param(option, default=''):
+    global _params
+    opt_type = _params[option]
+    param_value = get_git_config(option.lower().replace('_', '.'),
+                                 opt_type=opt_type, convert_type=False)
+    if param_value == '':
+        param_value = os.environ.get(option, default)
+    if opt_type == '--bool' and not isinstance(param_value, bool):
+        param_value = param_value.lower() == 'true'
+    elif param_value and opt_type == '--int':
+        param_value = int(param_value)
+    return param_value
+
+
 git_hook_file = """#!/usr/bin/env python
 import sys
-import os
-from flake8.hooks import git_hook
-
-COMPLEXITY = os.getenv('FLAKE8_COMPLEXITY', 10)
-STRICT = os.getenv('FLAKE8_STRICT', False)
-IGNORE = os.getenv('FLAKE8_IGNORE')
-LAZY = os.getenv('FLAKE8_LAZY', False)
+from flake8.hooks import git_hook, get_git_param
 
+# `get_git_param` will retrieve configuration from your local git config and
+# then fall back to using the environment variables that the hook has always
+# supported.
+# For example, to set the complexity, you'll need to do:
+#   git config flake8.complexity 10
+COMPLEXITY = get_git_param('FLAKE8_COMPLEXITY', 10)
+STRICT = get_git_param('FLAKE8_STRICT', False)
+IGNORE = get_git_param('FLAKE8_IGNORE', None)
+LAZY = get_git_param('FLAKE8_LAZY', False)
 
 if __name__ == '__main__':
     sys.exit(git_hook(
@@ -192,9 +235,10 @@ if __name__ == '__main__':
 
 
 def _install_hg_hook(path):
+    getenv = os.environ.get
     if not os.path.isfile(path):
         # Make the file so we can avoid IOError's
-        open(path, 'w+').close()
+        open(path, 'w').close()
 
     c = ConfigParser()
     c.readfp(open(path, 'r'))
@@ -211,18 +255,19 @@ def _install_hg_hook(path):
         c.add_section('flake8')
 
     if not c.has_option('flake8', 'complexity'):
-        c.set('flake8', 'complexity', str(os.getenv('FLAKE8_COMPLEXITY', 10)))
+        c.set('flake8', 'complexity', str(getenv('FLAKE8_COMPLEXITY', 10)))
 
     if not c.has_option('flake8', 'strict'):
-        c.set('flake8', 'strict', os.getenv('FLAKE8_STRICT', False))
+        c.set('flake8', 'strict', getenv('FLAKE8_STRICT', False))
 
     if not c.has_option('flake8', 'ignore'):
-        c.set('flake8', 'ignore', os.getenv('FLAKE8_IGNORE'))
+        c.set('flake8', 'ignore', getenv('FLAKE8_IGNORE', ''))
 
     if not c.has_option('flake8', 'lazy'):
-        c.set('flake8', 'lazy', os.getenv('FLAKE8_LAZY', False))
+        c.set('flake8', 'lazy', getenv('FLAKE8_LAZY', False))
 
-    c.write(open(path, 'w+'))
+    with open(path, 'w') as fd:
+        c.write(fd)
 
 
 def install_hook():
@@ -232,13 +277,15 @@ def install_hook():
         p = get_parser()[0]
         sys.stderr.write('Error: could not find either a git or mercurial '
                          'directory. Please re-run this in a proper '
-                         'repository.')
+                         'repository.\n')
         p.print_help()
         sys.exit(1)
 
     status = 0
     if 'git' in vcs:
-        with open(vcs, 'w+') as fd:
+        if os.path.exists(vcs):
+            sys.exit('Error: hook already exists (%s)' % vcs)
+        with open(vcs, 'w') as fd:
             fd.write(git_hook_file)
         # rwxr--r--
         os.chmod(vcs, stat.S_IRWXU | stat.S_IRGRP | stat.S_IROTH)
