@@ -1,4 +1,5 @@
 # -*- coding: utf-8 -*-
+import os
 
 try:
     import ast
@@ -37,12 +38,32 @@ class OrderedSet(list):
             self.append(value)
 
 
-def is_flag(val):
-    """Guess if the value could be an on/off toggle"""
-    val = str(val)
-    return val.upper() in ('1', '0', 'F', 'T', 'TRUE', 'FALSE', 'ON', 'OFF')
+def is_windows():
+    """Determine if the system is Windows."""
+    return os.name == 'nt'
 
 
-def flag_on(val):
-    """Return true if flag is on"""
-    return str(val).upper() in ('1', 'T', 'TRUE', 'ON')
+def is_using_stdin(paths):
+    """Determine if we're running checks on stdin."""
+    return '-' in paths
+
+
+def warn_when_using_jobs(options):
+    return (options.verbose and options.jobs and options.jobs.isdigit() and
+            int(options.jobs) > 1)
+
+
+def force_disable_jobs(styleguide):
+    return is_windows() or is_using_stdin(styleguide.paths)
+
+
+def option_normalizer(value):
+    if str(value).upper() in ('1', 'T', 'TRUE', 'ON'):
+        value = True
+    if str(value).upper() in ('0', 'F', 'FALSE', 'OFF'):
+        value = False
+
+    if isinstance(value, str):
+        value = [opt.strip() for opt in value.split(',') if opt.strip()]
+
+    return value
